/***
 * Copyright (C) 1996-98. by Miroslav Zagorac <zaga@tirnanog.cc.fer.hr>
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>

#include "../data/objects_def.h"
#include "../lib/lib_debug.h"

#include "cave.h"
#include "cave_data.h"
#include "main.h"
#include "objects.h"
#include "commodore.h"


struct OBJECT_DATA commodore_objects_data[] =
{
   {
      SPACE, SPACE, OBJECT_DESTROY_FLAG,
      {
         commodore_space,
         NULL
      }
   },
   {
      EXPLODE_S0, SPACE, 0,
      {
         commodore_explode_s0,
         commodore_explode_s1,
         commodore_explode_s1,
         commodore_explode_s0,
         commodore_explode_s0,
         commodore_explode_s2,
         NULL
      }
   },
   {
      EXPLODE_D0, DIAMOND, 0,
      {
         commodore_explode_d0,
         commodore_explode_d1,
         commodore_explode_d1,
         commodore_explode_d2,
         commodore_explode_d2,
         commodore_explode_d3,
         NULL
      }
   },
   {
      DIRT, DIRT, OBJECT_DESTROY_FLAG,
      {
         commodore_dirt,
         NULL
      }
   },
   {
      BRICK_WALL, BRICK_WALL, OBJECT_DROP_FLAG | OBJECT_DESTROY_FLAG,
      {
         commodore_brick_wall,
         NULL
      }
   },
   {
      MAGIC_WALL, MAGIC_WALL, OBJECT_DESTROY_FLAG,
      {
         commodore_magic_wall_0,
         commodore_magic_wall_0,
         commodore_magic_wall_1,
         commodore_magic_wall_1,
         commodore_magic_wall_2,
         commodore_magic_wall_2,
         commodore_magic_wall_3,
         commodore_magic_wall_3,
         NULL
      }
   },
   {
      WALL, WALL, OBJECT_DROP_FLAG,
      {
         commodore_steel_wall,
         NULL
      }
   },
   {
      DIAMOND, DIAMOND, OBJECT_FALL_FLAG | OBJECT_DROP_FLAG | OBJECT_DESTROY_FLAG,
      {
         commodore_diamond_0,
         commodore_diamond_1,
         commodore_diamond_2,
         commodore_diamond_3,
         commodore_diamond_4,
         commodore_diamond_5,
         commodore_diamond_6,
         commodore_diamond_7,
         NULL
      }
   },
   {
      DIAMOND_FALL, DIAMOND_FALL, OBJECT_FALL_FLAG | OBJECT_DROP_FLAG | OBJECT_DESTROY_FLAG,
      {
         commodore_diamond_0,
         commodore_diamond_1,
         commodore_diamond_2,
         commodore_diamond_3,
         commodore_diamond_4,
         commodore_diamond_5,
         commodore_diamond_6,
         commodore_diamond_7,
         NULL
      }
   },
   {
      FIREFLY_1, FIREFLY_1, OBJECT_DESTROY_FLAG,
      {
         commodore_firefly_0,
         commodore_firefly_0,
         commodore_firefly_1,
         commodore_firefly_1,
         commodore_firefly_2,
         commodore_firefly_2,
         commodore_firefly_3,
         commodore_firefly_3,
         NULL
      }
   },
   {
      BUTTERFLY_1, BUTTERFLY_1, OBJECT_DESTROY_FLAG,
      {
         commodore_butterfly_0,
         commodore_butterfly_0,
         commodore_butterfly_1,
         commodore_butterfly_2,
         commodore_butterfly_1,
         commodore_butterfly_0,
         commodore_butterfly_0,
         NULL
      }
   },
   {
      BOULDER, BOULDER, OBJECT_FALL_FLAG | OBJECT_DROP_FLAG | OBJECT_DESTROY_FLAG,
      {
         commodore_boulder,
         NULL
      }
   },
   {
      BOULDER_FALL, BOULDER_FALL, OBJECT_FALL_FLAG | OBJECT_DROP_FLAG | OBJECT_DESTROY_FLAG,
      {
         commodore_boulder,
         NULL
      }
   },
   {
      ROCKFORD, ROCKFORD, OBJECT_DESTROY_FLAG,
      {
         commodore_rockford_0,
         NULL
      }
   },
   {
      ROCKFORD_0, ROCKFORD_RANDOM, OBJECT_DESTROY_FLAG,
      {
         commodore_rockford_1,
         commodore_rockford_2,
         commodore_rockford_1,
         NULL
      }
   },
   {
      ROCKFORD_1, ROCKFORD_RANDOM, OBJECT_DESTROY_FLAG,
      {
         commodore_rockford_3,
         commodore_rockford_3,
         commodore_rockford_3,
         commodore_rockford_3,
         NULL
      }
   },
   {
      ROCKFORD_2, ROCKFORD_RANDOM, OBJECT_DESTROY_FLAG,
      {
         commodore_rockford_3,
         commodore_rockford_6, 
         commodore_rockford_6,
         commodore_rockford_3,
         commodore_rockford_3,
         commodore_rockford_6,
         commodore_rockford_6,
         commodore_rockford_3,
         commodore_rockford_3,
         commodore_rockford_6,
         commodore_rockford_6,
         commodore_rockford_3,
         NULL
      }
   },
   {
      ROCKFORD_3, ROCKFORD_RANDOM, OBJECT_DESTROY_FLAG,
      {
         commodore_rockford_3,
         commodore_rockford_3,
         commodore_rockford_4,
         commodore_rockford_5,
         commodore_rockford_4,
         commodore_rockford_3,
         commodore_rockford_3,
         NULL
      }
   },
   {
      ROCKFORD_R, ROCKFORD_R, OBJECT_DESTROY_FLAG,
      {
         commodore_rockford_r0,
         commodore_rockford_r1,
         commodore_rockford_r2,
         commodore_rockford_r3,
         commodore_rockford_r4,
         commodore_rockford_r5,
         commodore_rockford_r6,
         NULL
      }
   },
   {
      ROCKFORD_L, ROCKFORD_L, OBJECT_DESTROY_FLAG,
      {
         commodore_rockford_l0,
         commodore_rockford_l1,
         commodore_rockford_l2,
         commodore_rockford_l3,
         commodore_rockford_l4,
         commodore_rockford_l5,
         commodore_rockford_l6,
         NULL
      }
   },
   {
      PRE_OUTBOX, PRE_OUTBOX, OBJECT_DROP_FLAG | OBJECT_DESTROY_FLAG,
      {
         commodore_steel_wall,
         NULL
      }
   },
   {
      FLASH_OUTBOX, FLASH_OUTBOX, OBJECT_DESTROY_FLAG,
      {
         commodore_cave_out,
         commodore_cave_out,
         commodore_cave_out,
         commodore_cave_out,
         commodore_steel_wall,
         commodore_steel_wall,
         commodore_steel_wall,
         commodore_steel_wall,
         NULL
      }
   },
   {
      PRE_ROCKFORD_1, PRE_ROCKFORD_1, 0,
      {
         commodore_cave_out,
         NULL
      }
   },
   {
      PRE_ROCKFORD_2, ROCKFORD, 0,
      {
         commodore_steel_wall,
         commodore_steel_wall,
         commodore_steel_wall,
         commodore_steel_wall,
         commodore_cave_out,
         commodore_cave_out,
         commodore_cave_out,
         commodore_cave_out,
         commodore_steel_wall,
         commodore_steel_wall,
         commodore_steel_wall,
         commodore_steel_wall,
         commodore_cave_out,
         commodore_cave_out,
         commodore_cave_out,
         commodore_cave_out,
         commodore_steel_wall,
         commodore_steel_wall,
         commodore_steel_wall,
         commodore_steel_wall,
         commodore_cave_out,
         commodore_cave_out,
         commodore_cave_out,
         commodore_cave_out,
         commodore_explode_s2,
         commodore_explode_s2,
         commodore_explode_s0,
         commodore_explode_s0,
         commodore_explode_s1,
         commodore_explode_s1,
         NULL
      }
   },
   {
      AMOEBA, AMOEBA, OBJECT_DESTROY_FLAG,
      {
         commodore_amoeba_0,
         commodore_amoeba_0,
         commodore_amoeba_1,
         commodore_amoeba_1,
         commodore_amoeba_2,
         commodore_amoeba_2,
         commodore_amoeba_3,
         commodore_amoeba_3,
         NULL
      }
   },
   {
      WALL_PART, WALL_PART, 0,
      {
         commodore_wall_s0,
         commodore_wall_s1,
         commodore_wall_s2,
         commodore_wall_s3,
         commodore_wall_s4,
         commodore_wall_s5,
         commodore_wall_s6,
         commodore_wall_s7,
         NULL
      }
   },
   {
      PRE_MAGIC_WALL, PRE_MAGIC_WALL, OBJECT_DESTROY_FLAG,
      {
         commodore_brick_wall,
         NULL
      }
   },
   {
      POST_MAGIC_WALL, POST_MAGIC_WALL, OBJECT_DESTROY_FLAG,
      {
         commodore_brick_wall,
         NULL
      }
   },
   {
      UNKNOWN, UNKNOWN, 0,
      {
         unknown,
         NULL
      }
   }
};

unchar commodore_cave_raw_data[80][CAVE_RAW_MAXSIZE] =
{
   /***
    * Boulder Dash 1
    *
    * cave_1 (Cave A: Intro)
    */
   {
      0x01, 0x14, 0x0A, 0x0F, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0C, 0x0C, 0x0C, 0x0C, 0x0C, 0x96, 0x6E,
      0x46, 0x28, 0x1E, 0x08, 0x0B, 0x09, 0xD4, 0x20, 0x00, 0x10, 0x14, 0x00, 0x3C, 0x32, 0x09, 0x00,
      0x42, 0x01, 0x09, 0x1E, 0x02, 0x42, 0x09, 0x10, 0x1E, 0x02, 0x25, 0x03, 0x04, 0x04, 0x26, 0x12,
      0xFF
   },
   /* cave_2 (Cave B: Rooms) */
   {
      0x02, 0x14, 0x14, 0x32, 0x03, 0x00, 0x01, 0x57, 0x58, 0x0A, 0x0C, 0x09, 0x0D, 0x0A, 0x96, 0x6E,
      0x46, 0x46, 0x46, 0x0A, 0x04, 0x09, 0x00, 0x00, 0x00, 0x10, 0x14, 0x08, 0x3C, 0x32, 0x09, 0x02,
      0x42, 0x01, 0x08, 0x26, 0x02, 0x42, 0x01, 0x0F, 0x26, 0x02, 0x42, 0x08, 0x03, 0x14, 0x04, 0x42,
      0x10, 0x03, 0x14, 0x04, 0x42, 0x18, 0x03, 0x14, 0x04, 0x42, 0x20, 0x03, 0x14, 0x04, 0x40, 0x01,
      0x05, 0x26, 0x02, 0x40, 0x01, 0x0B, 0x26, 0x02, 0x40, 0x01, 0x12, 0x26, 0x02, 0x40, 0x14, 0x03,
      0x14, 0x04, 0x25, 0x12, 0x15, 0x04, 0x12, 0x16, 0xFF
   },
   /* cave_3 (Cave C: Maze) */
   {
      0x03, 0x00, 0x0F, 0x00, 0x00, 0x32, 0x36, 0x34, 0x37, 0x18, 0x17, 0x18, 0x17, 0x15, 0x96, 0x64,
      0x5A, 0x50, 0x46, 0x09, 0x08, 0x09, 0x04, 0x00, 0x02, 0x10, 0x14, 0x00, 0x64, 0x32, 0x09, 0x00,
      0x25, 0x03, 0x04, 0x04, 0x27, 0x14, 0xFF
   },
   /* cave_4 (Cave D: Butterflies) */
   {
      0x04, 0x14, 0x05, 0x14, 0x00, 0x6E, 0x70, 0x73, 0x77, 0x24, 0x24, 0x24, 0x24, 0x24, 0x78, 0x64,
      0x50, 0x3C, 0x32, 0x04, 0x08, 0x09, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x14, 0x00, 0x00, 0x00,
      0x25, 0x01, 0x03, 0x04, 0x26, 0x16, 0x81, 0x08, 0x0A, 0x04, 0x04, 0x00, 0x30, 0x0A, 0x0B, 0x81,
      0x10, 0x0A, 0x04, 0x04, 0x00, 0x30, 0x12, 0x0B, 0x81, 0x18, 0x0A, 0x04, 0x04, 0x00, 0x30, 0x1A,
      0x0B, 0x81, 0x20, 0x0A, 0x04, 0x04, 0x00, 0x30, 0x22, 0x0B, 0xFF
   },
   /* cave_5 (Cave E: Guards) */
   {
      0x05, 0x14, 0x32, 0x5A, 0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0x05, 0x06, 0x07, 0x08, 0x96, 0x78,
      0x5A, 0x3C, 0x1E, 0x09, 0x0A, 0x09, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x25, 0x01, 0x03, 0x04, 0x27, 0x16, 0x80, 0x08, 0x0A, 0x03, 0x03, 0x00, 0x80, 0x10, 0x0A, 0x03,
      0x03, 0x00, 0x80, 0x18, 0x0A, 0x03, 0x03, 0x00, 0x80, 0x20, 0x0A, 0x03, 0x03, 0x00, 0x14, 0x09,
      0x0C, 0x08, 0x0A, 0x0A, 0x14, 0x11, 0x0C, 0x08, 0x12, 0x0A, 0x14, 0x19, 0x0C, 0x08, 0x1A, 0x0A,
      0x14, 0x21, 0x0C, 0x08, 0x22, 0x0A, 0x80, 0x08, 0x10, 0x03, 0x03, 0x00, 0x80, 0x10, 0x10, 0x03,
      0x03, 0x00, 0x80, 0x18, 0x10, 0x03, 0x03, 0x00, 0x80, 0x20, 0x10, 0x03, 0x03, 0x00, 0x14, 0x09,
      0x12, 0x08, 0x0A, 0x10, 0x14, 0x11, 0x12, 0x08, 0x12, 0x10, 0x14, 0x19, 0x12, 0x08, 0x1A, 0x10,
      0x14, 0x21, 0x12, 0x08, 0x22, 0x10, 0xFF
   },
   /* cave_6 (Cave F: Firefly dens) */
   {
      0x06, 0x14, 0x28, 0x3C, 0x00, 0x14, 0x15, 0x16, 0x17, 0x04, 0x06, 0x07, 0x08, 0x08, 0x96, 0x78,
      0x64, 0x5A, 0x50, 0x0E, 0x0A, 0x09, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x32, 0x00, 0x00, 0x00,
      0x82, 0x01, 0x03, 0x0A, 0x04, 0x00, 0x82, 0x01, 0x06, 0x0A, 0x04, 0x00, 0x82, 0x01, 0x09, 0x0A,
      0x04, 0x00, 0x82, 0x01, 0x0C, 0x0A, 0x04, 0x00, 0x41, 0x0A, 0x03, 0x0D, 0x04, 0x14, 0x03, 0x05,
      0x08, 0x04, 0x05, 0x14, 0x03, 0x08, 0x08, 0x04, 0x08, 0x14, 0x03, 0x0B, 0x08, 0x04, 0x0B, 0x14,
      0x03, 0x0E, 0x08, 0x04, 0x0E, 0x82, 0x1D, 0x03, 0x0A, 0x04, 0x00, 0x82, 0x1D, 0x06, 0x0A, 0x04,
      0x00, 0x82, 0x1D, 0x09, 0x0A, 0x04, 0x00, 0x82, 0x1D, 0x0C, 0x0A, 0x04, 0x00, 0x41, 0x1D, 0x03,
      0x0D, 0x04, 0x14, 0x24, 0x05, 0x08, 0x23, 0x05, 0x14, 0x24, 0x08, 0x08, 0x23, 0x08, 0x14, 0x24,
      0x0B, 0x08, 0x23, 0x0B, 0x14, 0x24, 0x0E, 0x08, 0x23, 0x0E, 0x25, 0x03, 0x14, 0x04, 0x26, 0x14,
      0xFF
   },
   /* cave_7 (Cave G: Amoeba) */
   {
      0x07, 0x4B, 0x0A, 0x14, 0x02, 0x07, 0x08, 0x0A, 0x09, 0x0F, 0x14, 0x19, 0x19, 0x19, 0x78, 0x78,
      0x78, 0x78, 0x78, 0x09, 0x0A, 0x0D, 0x00, 0x00, 0x00, 0x10, 0x08, 0x00, 0x64, 0x28, 0x02, 0x00,
      0x42, 0x01, 0x07, 0x0C, 0x02, 0x42, 0x1C, 0x05, 0x0B, 0x02, 0x7A, 0x13, 0x15, 0x02, 0x02, 0x14,
      0x04, 0x06, 0x14, 0x04, 0x0E, 0x14, 0x04, 0x16, 0x14, 0x22, 0x04, 0x14, 0x22, 0x0C, 0x14, 0x22,
      0x16, 0x25, 0x14, 0x03, 0x04, 0x27, 0x07, 0xFF
   },
   /* cave_8 (Cave H: Enchanted wall) */
   {
      0x08, 0x14, 0x0A, 0x14, 0x01, 0x03, 0x04, 0x05, 0x06, 0x0A, 0x0F, 0x14, 0x14, 0x14, 0x78, 0x6E,
      0x64, 0x5A, 0x50, 0x02, 0x0E, 0x09, 0x00, 0x00, 0x00, 0x10, 0x08, 0x00, 0x5A, 0x32, 0x02, 0x00,
      0x14, 0x04, 0x06, 0x14, 0x22, 0x04, 0x14, 0x22, 0x0C, 0x04, 0x00, 0x05, 0x25, 0x14, 0x03, 0x42,
      0x01, 0x07, 0x0C, 0x02, 0x42, 0x01, 0x0F, 0x0C, 0x02, 0x42, 0x1C, 0x05, 0x0B, 0x02, 0x42, 0x1C,
      0x0D, 0x0B, 0x02, 0x43, 0x0E, 0x11, 0x08, 0x02, 0x14, 0x0C, 0x10, 0x00, 0x0E, 0x12, 0x14, 0x13,
      0x12, 0x41, 0x0E, 0x0F, 0x08, 0x02, 0xFF
   },
   /* cave_9 (Cave I: Greed) */
   {
      0x09, 0x14, 0x05, 0x0A, 0x64, 0x89, 0x8C, 0xFB, 0x33, 0x4B, 0x4B, 0x50, 0x55, 0x5A, 0x96, 0x96,
      0x82, 0x82, 0x78, 0x08, 0x04, 0x09, 0x00, 0x00, 0x10, 0x14, 0x00, 0x00, 0xF0, 0x78, 0x00, 0x00,
      0x82, 0x05, 0x0A, 0x0D, 0x0D, 0x00, 0x01, 0x0C, 0x0A, 0x82, 0x19, 0x0A, 0x0D, 0x0D, 0x00, 0x01,
      0x1F, 0x0A, 0x42, 0x11, 0x12, 0x09, 0x02, 0x40, 0x11, 0x13, 0x09, 0x02, 0x25, 0x07, 0x0C, 0x04,
      0x08, 0x0C, 0xFF
   },
   /* cave_10 (Cave J: Tracks) */
   {
      0x0A, 0x14, 0x19, 0x3C, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0C, 0x0C, 0x0C, 0x0C, 0x0C, 0x96, 0x82,
      0x78, 0x6E, 0x64, 0x06, 0x08, 0x09, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x25, 0x0D, 0x03, 0x04, 0x27, 0x16, 0x54, 0x05, 0x04, 0x11, 0x03, 0x54, 0x15, 0x04, 0x11, 0x05,
      0x80, 0x05, 0x0B, 0x11, 0x03, 0x08, 0xC2, 0x01, 0x04, 0x15, 0x11, 0x00, 0x0D, 0x04, 0xC2, 0x07,
      0x06, 0x0D, 0x0D, 0x00, 0x0D, 0x06, 0xC2, 0x09, 0x08, 0x09, 0x09, 0x00, 0x0D, 0x08, 0xC2, 0x0B,
      0x0A, 0x05, 0x05, 0x00, 0x0D, 0x0A, 0x82, 0x03, 0x06, 0x03, 0x0F, 0x08, 0x00, 0x04, 0x06, 0x54,
      0x04, 0x10, 0x04, 0x04, 0xFF
   },
   /* cave_11 (Cave K: Crowd) */
   {
      0x0B, 0x14, 0x32, 0x00, 0x00, 0x04, 0x66, 0x97, 0x64, 0x06, 0x06, 0x06, 0x06, 0x06, 0x78, 0x78,
      0x96, 0x96, 0xF0, 0x0B, 0x08, 0x09, 0x00, 0x00, 0x00, 0x10, 0x08, 0x00, 0x64, 0x50, 0x02, 0x00,
      0x42, 0x0A, 0x03, 0x09, 0x04, 0x42, 0x14, 0x03, 0x09, 0x04, 0x42, 0x1E, 0x03, 0x09, 0x04, 0x42,
      0x09, 0x16, 0x09, 0x00, 0x42, 0x0C, 0x0F, 0x11, 0x02, 0x42, 0x05, 0x0B, 0x09, 0x02, 0x42, 0x0F,
      0x0B, 0x09, 0x02, 0x42, 0x19, 0x0B, 0x09, 0x02, 0x42, 0x1C, 0x13, 0x0B, 0x01, 0x14, 0x04, 0x03,
      0x14, 0x0E, 0x03, 0x14, 0x18, 0x03, 0x14, 0x22, 0x03, 0x14, 0x04, 0x16, 0x14, 0x23, 0x15, 0x25,
      0x14, 0x14, 0x04, 0x26, 0x11, 0xFF
   },
   /* cave_12 (Cave L: Walls) */
   {
      0x0C, 0x14, 0x14, 0x00, 0x00, 0x3C, 0x02, 0x3B, 0x66, 0x13, 0x13, 0x0E, 0x10, 0x15, 0xB4, 0xAA,
      0xA0, 0xA0, 0xA0, 0x0C, 0x0A, 0x09, 0x00, 0x00, 0x00, 0x10, 0x14, 0x00, 0x3C, 0x32, 0x09, 0x00,
      0x42, 0x0A, 0x05, 0x12, 0x04, 0x42, 0x0E, 0x05, 0x12, 0x04, 0x42, 0x12, 0x05, 0x12, 0x04, 0x42,
      0x16, 0x05, 0x12, 0x04, 0x42, 0x02, 0x06, 0x0B, 0x02, 0x42, 0x02, 0x0A, 0x0B, 0x02, 0x42, 0x02,
      0x0E, 0x0F, 0x02, 0x42, 0x02, 0x12, 0x0B, 0x02, 0x81, 0x1E, 0x04, 0x04, 0x04, 0x00, 0x08, 0x20,
      0x05, 0x81, 0x1E, 0x09, 0x04, 0x04, 0x00, 0x08, 0x20, 0x0A, 0x81, 0x1E, 0x0E, 0x04, 0x04, 0x00,
      0x08, 0x20, 0x0F, 0x25, 0x03, 0x14, 0x04, 0x27, 0x16, 0xFF
   },
   /* cave_13 (Cave M: Apocalypse) */
   {
      0x0D, 0x8C, 0x05, 0x08, 0x00, 0x01, 0x02, 0x03, 0x04, 0x32, 0x37, 0x3C, 0x46, 0x50, 0xA0, 0x9B,
      0x96, 0x91, 0x8C, 0x06, 0x08, 0x0D, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x28, 0x00, 0x00, 0x00,
      0x25, 0x12, 0x03, 0x04, 0x0A, 0x03, 0x3A, 0x14, 0x03, 0x42, 0x05, 0x12, 0x1E, 0x02, 0x70, 0x05,
      0x13, 0x1E, 0x02, 0x50, 0x05, 0x14, 0x1E, 0x02, 0xC1, 0x05, 0x15, 0x1E, 0x02, 0xFF
   },
   /* cave_14 (Cave N: Zigzag) */
   {
      0x0E, 0x14, 0x0A, 0x14, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1E, 0x23, 0x28, 0x2A, 0x2D, 0x96, 0x91,
      0x8C, 0x87, 0x82, 0x0C, 0x08, 0x09, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x81, 0x0A, 0x0A, 0x0D, 0x0D, 0x00, 0x70, 0x0B, 0x0B, 0x0C, 0x03, 0xC1, 0x0C, 0x0A, 0x03, 0x0D,
      0xC1, 0x10, 0x0A, 0x03, 0x0D, 0xC1, 0x14, 0x0A, 0x03, 0x0D, 0x50, 0x16, 0x08, 0x0C, 0x02, 0x48,
      0x16, 0x07, 0x0C, 0x02, 0xC1, 0x17, 0x06, 0x03, 0x04, 0xC1, 0x1B, 0x06, 0x03, 0x04, 0xC1, 0x1F,
      0x06, 0x03, 0x04, 0x25, 0x03, 0x03, 0x04, 0x27, 0x14, 0xFF
   },
   /* cave_15 (Cave O: Funnel) */
   {
      0x0F, 0x08, 0x0A, 0x14, 0x01, 0x1D, 0x1E, 0x1F, 0x20, 0x0F, 0x14, 0x14, 0x19, 0x1E, 0x78, 0x78,
      0x78, 0x78, 0x8C, 0x08, 0x0E, 0x09, 0x00, 0x00, 0x00, 0x10, 0x08, 0x00, 0x64, 0x50, 0x02, 0x00,
      0x42, 0x02, 0x04, 0x0A, 0x03, 0x42, 0x0F, 0x0D, 0x0A, 0x01, 0x41, 0x0C, 0x0E, 0x03, 0x02, 0x43,
      0x0C, 0x0F, 0x03, 0x02, 0x04, 0x14, 0x16, 0x25, 0x14, 0x03, 0xFF
   },
   /* cave_16 (Cave P: Enchanted boxes) */
   {
      0x10, 0x14, 0x0A, 0x14, 0x01, 0x78, 0x81, 0x7E, 0x7B, 0x0C, 0x0F, 0x0F, 0x0F, 0x0C, 0x96, 0x96,
      0x96, 0x96, 0x96, 0x09, 0x0A, 0x09, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x32, 0x00, 0x00, 0x00,
      0x25, 0x01, 0x03, 0x04, 0x27, 0x04, 0x81, 0x08, 0x13, 0x04, 0x04, 0x00, 0x08, 0x0A, 0x14, 0xC2,
      0x07, 0x0A, 0x06, 0x08, 0x43, 0x07, 0x0A, 0x06, 0x02, 0x81, 0x10, 0x13, 0x04, 0x04, 0x00, 0x08,
      0x12, 0x14, 0xC2, 0x0F, 0x0A, 0x06, 0x08, 0x43, 0x0F, 0x0A, 0x06, 0x02, 0x81, 0x18, 0x13, 0x04,
      0x04, 0x00, 0x08, 0x1A, 0x14, 0x81, 0x20, 0x13, 0x04, 0x04, 0x00, 0x08, 0x22, 0x14, 0xFF
   },
   /* cave_17 (Intermission 1) */
   {
      0x11, 0x14, 0x1E, 0x00, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x06, 0x06, 0x06, 0x06, 0x06, 0x0A, 0x0A,
      0x0A, 0x0A, 0x0A, 0x0E, 0x02, 0x09, 0x00, 0x00, 0x00, 0x14, 0x00, 0x00, 0xFF, 0x09, 0x00, 0x00,
      0x87, 0x00, 0x02, 0x28, 0x16, 0x07, 0x87, 0x00, 0x02, 0x14, 0x0C, 0x00, 0x32, 0x0A, 0x0C, 0x10,
      0x0A, 0x04, 0x01, 0x0A, 0x05, 0x25, 0x03, 0x05, 0x04, 0x12, 0x0C, 0xFF
   },
   /* cave_18 (Intermission 2) */
   {
      0x12, 0x14, 0x0A, 0x00, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x10, 0x10, 0x10, 0x10, 0x10, 0x0F, 0x0F,
      0x0F, 0x0F, 0x0F, 0x06, 0x0F, 0x09, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x87, 0x00, 0x02, 0x28, 0x16, 0x07, 0x87, 0x00, 0x02, 0x14, 0x0C, 0x01, 0x50, 0x01, 0x03, 0x09,
      0x03, 0x48, 0x02, 0x03, 0x08, 0x03, 0x54, 0x01, 0x05, 0x08, 0x03, 0x50, 0x01, 0x06, 0x07, 0x03,
      0x50, 0x12, 0x03, 0x09, 0x05, 0x54, 0x12, 0x05, 0x08, 0x05, 0x50, 0x12, 0x06, 0x07, 0x05, 0x25,
      0x01, 0x04, 0x04, 0x12, 0x04, 0xFF
   },
   /* cave_19 (Intermission 3) */
   {
      0x13, 0x04, 0x0A, 0x00, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x14, 0x14,
      0x14, 0x14, 0x14, 0x06, 0x08, 0x09, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x87, 0x00, 0x02, 0x28, 0x16, 0x07, 0x87, 0x00, 0x02, 0x14, 0x0C, 0x00, 0x54, 0x01, 0x0C, 0x12,
      0x02, 0x88, 0x0F, 0x09, 0x04, 0x04, 0x08, 0x25, 0x08, 0x03, 0x04, 0x12, 0x07, 0xFF
   },
   /* cave_20 (Intermission 4) */
   {
      0x14, 0x03, 0x1E, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x06, 0x06, 0x06, 0x06, 0x06, 0x14, 0x14,
      0x14, 0x14, 0x14, 0x06, 0x08, 0x09, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x87, 0x00, 0x02, 0x28, 0x16, 0x07, 0x87, 0x00, 0x02, 0x14, 0x0C, 0x01, 0xD0, 0x0B, 0x03, 0x03,
      0x02, 0x80, 0x0B, 0x07, 0x03, 0x06, 0x00, 0x43, 0x0B, 0x06, 0x03, 0x02, 0x43, 0x0B, 0x0A, 0x03,
      0x02, 0x50, 0x08, 0x07, 0x03, 0x03, 0x25, 0x03, 0x03, 0x04, 0x09, 0x0A, 0xFF
   },

   /***
    * Boulder Dash 2
    */
   { 0x00 },
   { 0x00 },
   { 0x00 },
   { 0x00 },
   { 0x00 },
   { 0x00 },
   { 0x00 },
   { 0x00 },
   { 0x00 },
   { 0x00 },
   { 0x00 },
   { 0x00 },
   { 0x00 },
   { 0x00 },
   { 0x00 },
   { 0x00 },
   { 0x00 },
   { 0x00 },
   { 0x00 },
   { 0x00 },

   /***
    * Boulder Dash 3
    */
   {
      0x01, 0x14, 0x0a, 0x0f, 0x0a, 0xbb, 0x0c, 0xad, 0x0e, 0x50, 0x50, 0x50, 0x50, 0x50, 0x96, 0x6e,
      0x6e, 0x6e, 0x6e, 0x0b, 0x0c, 0x09, 0x00, 0x47, 0x01, 0x00, 0x10, 0x14, 0xfe, 0x40, 0x40, 0x25,
      0x42, 0x01, 0x04, 0x0d, 0x03, 0x42, 0x1f, 0x03, 0x0a, 0x05, 0x42, 0x01, 0x14, 0x20, 0x02, 0xc2,
      0x0e, 0x09, 0x05, 0x05, 0xc2, 0x1b, 0x09, 0x08, 0x07, 0x01, 0x10, 0x0d, 0x01, 0x1d, 0x0f, 0x25,
      0x10, 0x0a, 0x04, 0x08, 0x17, 0xff
   },

   {
      0x02, 0x14, 0x0f, 0x14, 0x07, 0x80, 0x90, 0xa0, 0xb0, 0x32, 0x32, 0x32, 0x32, 0x32, 0x96, 0x6e,
      0x6e, 0x6e, 0x6e, 0x02, 0x0a, 0x09, 0x42, 0x42, 0x01, 0x00, 0x10, 0x00, 0x10, 0x20, 0x20, 0x00,
      0xc2, 0x02, 0x0a, 0x06, 0x07, 0xc2, 0x09, 0x0a, 0x05, 0x07, 0xc2, 0x12, 0x0a, 0x05, 0x07, 0xc2,
      0x18, 0x0a, 0x08, 0x07, 0xe4, 0x03, 0x0b, 0x04, 0x05, 0xe4, 0x0a, 0x0b, 0x03, 0x05, 0xe4, 0x13,
      0x0b, 0x03, 0x05, 0xe4, 0x19, 0x0b, 0x06, 0x05, 0x48, 0x11, 0x03, 0x06, 0x04, 0x25, 0x05, 0x17,
      0x04, 0x04, 0x17, 0xff
   },

   {
      0x03, 0x40, 0x0a, 0x0f, 0x01, 0x22, 0x03, 0x04, 0x05, 0x28, 0x28, 0x28, 0x28, 0x28, 0xc9, 0x96,
      0x96, 0x6e, 0x6e, 0x06, 0x16, 0x16, 0x00, 0x06, 0x00, 0x03, 0x10, 0x14, 0x30, 0x70, 0x40, 0x10,
      0x04, 0x04, 0x04, 0x25, 0x05, 0x05, 0xff
   },

   {
      0x04, 0x4b, 0x0f, 0x14, 0xa0, 0xb0, 0xc0, 0xd0, 0xe0, 0x4b, 0x4b, 0x4b, 0x4b, 0x4b, 0xc9, 0xc9,
      0x96, 0x96, 0x6e, 0x02, 0x1f, 0x1f, 0x03, 0x03, 0x00, 0x10, 0x00, 0x00, 0x20, 0x30, 0x00, 0x00,
      0x50, 0x09, 0x03, 0x0c, 0x03, 0x50, 0x07, 0x04, 0x0c, 0x03, 0x70, 0x08, 0x03, 0x0c, 0x03, 0x70,
      0x08, 0x05, 0x0c, 0x03, 0x42, 0x07, 0x05, 0x0c, 0x03, 0x50, 0x23, 0x03, 0x0c, 0x05, 0x70, 0x24,
      0x03, 0x0c, 0x05, 0x42, 0x24, 0x04, 0x0c, 0x05, 0x3a, 0x0c, 0x0c, 0x25, 0x03, 0x03, 0x04, 0x07,
      0x03, 0xff
   },

   {
      0x05, 0x14, 0x14, 0x19, 0x11, 0x22, 0x33, 0x44, 0x55, 0x14, 0x14, 0x14, 0x14, 0x14, 0x96, 0x6e,
      0x6e, 0x6e, 0x6e, 0x04, 0x27, 0x27, 0x04, 0x04, 0x00, 0x12, 0x08, 0x14, 0x20, 0x30, 0x10, 0x0a,
      0x25, 0x26, 0x10, 0x04, 0x04, 0x04, 0xff
   },

   {
      0x06, 0x14, 0x0a, 0x19, 0x12, 0x34, 0x56, 0x78, 0x9a, 0x96, 0x96, 0x96, 0x96, 0x96, 0x96, 0x64,
      0x64, 0x4b, 0x4b, 0x47, 0x07, 0x07, 0x47, 0x47, 0x00, 0x10, 0x08, 0x14, 0x50, 0x40, 0x38, 0x25,
      0x25, 0x24, 0x11, 0x04, 0x24, 0x10, 0xff
   },

   {
      0x07, 0x43, 0x0a, 0x0f, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x19, 0x19, 0x19, 0x19, 0x19, 0x96, 0x96,
      0x96, 0x96, 0x96, 0x41, 0x0f, 0x0f, 0x41, 0x41, 0x00, 0x00, 0x10, 0x08, 0x00, 0x00, 0x20, 0x02,
      0x42, 0x01, 0x0c, 0x10, 0x02, 0x48, 0x01, 0x0d, 0x10, 0x02, 0x50, 0x01, 0x0e, 0x10, 0x02, 0x50,
      0x15, 0x03, 0x03, 0x04, 0xfa, 0x15, 0x03, 0x02, 0x10, 0xd4, 0x1a, 0x0a, 0x06, 0x06, 0xd4, 0x1b,
      0x0b, 0x04, 0x04, 0xd4, 0x1c, 0x0c, 0x02, 0x02, 0x25, 0x02, 0x04, 0x04, 0x25, 0x04, 0xff
   },

   {
      0x08, 0x20, 0x0a, 0x0f, 0xaa, 0xbb, 0xcc, 0x0d, 0xee, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x96, 0x96,
      0x6e, 0x6e, 0x56, 0x02, 0x37, 0x16, 0x06, 0x06, 0x10, 0x00, 0x00, 0x00, 0x30, 0x00, 0x00, 0x00,
      0x43, 0x07, 0x08, 0x1d, 0x02, 0x43, 0x07, 0x0b, 0x1d, 0x02, 0x43, 0x07, 0x0f, 0x1d, 0x02, 0x25,
      0x01, 0x03, 0x04, 0x02, 0x17, 0xff
   },

   {
      0x09, 0x14, 0x0a, 0x0f, 0x31, 0x25, 0x2d, 0x34, 0x56, 0x5a, 0x5a, 0x5a, 0x5a, 0x5a, 0x96, 0x96,
      0x6e, 0x6e, 0x6e, 0x47, 0x07, 0x07, 0x47, 0x47, 0x00, 0x10, 0x14, 0x00, 0x00, 0x50, 0x35, 0x00,
      0x42, 0x03, 0x03, 0x10, 0x04, 0x42, 0x06, 0x16, 0x10, 0x00, 0x42, 0x09, 0x03, 0x10, 0x04, 0x42,
      0x0c, 0x16, 0x10, 0x00, 0x42, 0x0f, 0x03, 0x10, 0x04, 0x42, 0x12, 0x03, 0x10, 0x04, 0x42, 0x15,
      0x16, 0x10, 0x00, 0x42, 0x19, 0x03, 0x10, 0x04, 0x42, 0x1c, 0x16, 0x10, 0x00, 0x42, 0x20, 0x03,
      0x10, 0x04, 0x25, 0x02, 0x04, 0x04, 0x25, 0x04, 0xff
   },

   {
      0x0a, 0x00, 0x19, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x96, 0x96,
      0x6e, 0x6e, 0x6e, 0x42, 0x17, 0x17, 0x42, 0x42, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x40, 0x0b, 0x06, 0x0c, 0x04, 0x40, 0x0a, 0x06, 0x0c, 0x04, 0x40, 0x0d, 0x06, 0x0c, 0x04, 0x40,
      0x0f, 0x06, 0x0c, 0x04, 0x40, 0x11, 0x06, 0x0c, 0x04, 0x40, 0x13, 0x06, 0x0c, 0x04, 0xc1, 0x0a,
      0x0e, 0x0b, 0x02, 0x42, 0x0a, 0x0c, 0x0b, 0x02, 0x54, 0x0a, 0x0b, 0x0b, 0x02, 0x54, 0x0a, 0x0d,
      0x0b, 0x02, 0x48, 0x0a, 0x08, 0x0b, 0x02, 0x48, 0x0a, 0x0f, 0x0b, 0x02, 0x42, 0x0a, 0x06, 0x0c,
      0x04, 0x42, 0x0a, 0x06, 0x0c, 0x04, 0x42, 0x0c, 0x06, 0x0c, 0x04, 0x42, 0x0e, 0x06, 0x0c, 0x04,
      0x42, 0x10, 0x06, 0x0c, 0x04, 0x42, 0x12, 0x06, 0x0c, 0x04, 0x42, 0x14, 0x06, 0x0c, 0x04, 0x50,
      0x16, 0x09, 0x0f, 0x02, 0x25, 0x02, 0x03, 0x04, 0x26, 0x14, 0xff
   },

   {
      0x0b, 0x30, 0x0a, 0x0f, 0x98, 0x87, 0x88, 0x65, 0x54, 0x19, 0x19, 0x19, 0x19, 0x19, 0x96, 0x96,
      0x6e, 0x6e, 0x6e, 0x06, 0x16, 0x16, 0x16, 0x06, 0x00, 0x10, 0x08, 0x00, 0x00, 0x30, 0x05, 0x00,
      0x3a, 0x12, 0x0d, 0xc2, 0x11, 0x0c, 0x05, 0x05, 0x25, 0x03, 0x03, 0x04, 0x26, 0x16, 0xff, 0x00,
      0x17, 0xff
   },

   {
      0x0c, 0x30, 0x0a, 0x0f, 0x56, 0x78, 0x6c, 0x98, 0xab, 0x19, 0x19, 0x19, 0x19, 0x19, 0x96, 0x96,
      0x96, 0x6e, 0x6e, 0x02, 0x1f, 0x1f, 0x03, 0x03, 0x10, 0x00, 0x00, 0x00, 0x45, 0x00, 0x00, 0x00,
      0x25, 0x0e, 0x0d, 0x08, 0x11, 0x0c, 0xfa, 0x0c, 0x0a, 0x05, 0x06, 0x50, 0x0a, 0x10, 0x0b, 0x02,
      0x04, 0x27, 0x14, 0xff
   },

   {
      0x0d, 0x8a, 0x0a, 0x0f, 0x56, 0x88, 0x90, 0x12, 0xab, 0x5f, 0x5f, 0x5f, 0x5f, 0x5f, 0x96, 0x96,
      0x6e, 0x6e, 0x6e, 0x04, 0x27, 0x27, 0x04, 0x04, 0x10, 0x00, 0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0xd0, 0x0f, 0x07, 0x05, 0x05, 0x3a, 0x12, 0x09, 0x40, 0x02, 0x05, 0x0b, 0x03, 0x70, 0x02, 0x04,
      0x0b, 0x03, 0x42, 0x02, 0x03, 0x0b, 0x03, 0x40, 0x25, 0x05, 0x0b, 0x05, 0x70, 0x25, 0x04, 0x0b,
      0x05, 0x42, 0x25, 0x03, 0x0b, 0x05, 0x25, 0x04, 0x17, 0x04, 0x27, 0x15, 0xff
   },

   {
      0x0e, 0x5e, 0x0a, 0x0f, 0xaa, 0x66, 0x57, 0xc6, 0x56, 0x55, 0x55, 0x55, 0x55, 0x55, 0x96, 0x96,
      0x6e, 0x6e, 0x6e, 0x47, 0x07, 0x07, 0x47, 0x47, 0x10, 0x00, 0x00, 0x00, 0x15, 0x00, 0x00, 0x00,
      0xc8, 0x04, 0x04, 0x09, 0x09, 0x3a, 0x08, 0x08, 0x42, 0x25, 0x04, 0x13, 0x04, 0x42, 0x23, 0x15,
      0x13, 0x00, 0x70, 0x26, 0x03, 0x14, 0x04, 0x70, 0x24, 0x03, 0x14, 0x04, 0x00, 0x25, 0x03, 0x25,
      0x04, 0x16, 0x04, 0x27, 0x15, 0xff
   },

   {
      0x0f, 0x3a, 0x05, 0x0a, 0xaa, 0xbb, 0xcc, 0xdd, 0xee, 0x5f, 0x5f, 0x5f, 0x5f, 0x5f, 0x96, 0x96,
      0x6e, 0x6e, 0x6e, 0x41, 0x0f, 0x0f, 0x41, 0x41, 0x10, 0x00, 0x00, 0x00, 0x25, 0x00, 0x00, 0x00,
      0x43, 0x03, 0x0c, 0x20, 0x02, 0x3a, 0x10, 0x16, 0x04, 0x01, 0x03, 0x25, 0x02, 0x03, 0xff
   },

   {
      0x10, 0x20, 0x0a, 0x0f, 0x44, 0x55, 0x66, 0x77, 0x88, 0x5f, 0x5f, 0x5f, 0x5f, 0x5f, 0x96, 0x96,
      0x6e, 0x6e, 0x6e, 0x02, 0x37, 0x16, 0x06, 0x06, 0x10, 0x14, 0x00, 0x00, 0x50, 0x30, 0x00, 0x00,
      0x43, 0x01, 0x0c, 0x25, 0x02, 0x40, 0x01, 0x0d, 0x25, 0x02, 0x48, 0x01, 0x0e, 0x25, 0x02, 0x25,
      0x01, 0x03, 0x04, 0x27, 0x16, 0xff
   },

   {
      0x11, 0x10, 0x0f, 0x19, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x1e, 0x1e, 0x1e, 0x1e, 0x1e, 0x1e, 0x1e,
      0x1e, 0x1e, 0x1e, 0x41, 0x2f, 0x0f, 0x4f, 0x4f, 0x00, 0x00, 0x00, 0x00, 0xff, 0x00, 0x00, 0x00,
      0x87, 0x00, 0x02, 0x28, 0x16, 0x07, 0x87, 0x00, 0x02, 0x14, 0x0c, 0x00, 0x10, 0x09, 0x03, 0x01,
      0x09, 0x04, 0x03, 0x09, 0x05, 0x3a, 0x09, 0x0c, 0x25, 0x01, 0x03, 0x04, 0x02, 0x03, 0xff
   },

   {
      0x12, 0x14, 0x0a, 0x14, 0xaa, 0xab, 0xac, 0xad, 0xae, 0x63, 0x63, 0x63, 0x63, 0x63, 0x1e, 0x1e,
      0x1e, 0x1e, 0x1e, 0x41, 0x2f, 0x0f, 0x4f, 0x4f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x87, 0x00, 0x02, 0x28, 0x16, 0x07, 0x87, 0x00, 0x02, 0x14, 0x0c, 0x00, 0x54, 0x01, 0x03, 0x12,
      0x02, 0x54, 0x01, 0x04, 0x12, 0x02, 0x54, 0x01, 0x05, 0x12, 0x02, 0x54, 0x01, 0x06, 0x12, 0x02,
      0x54, 0x01, 0x07, 0x12, 0x02, 0x54, 0x01, 0x08, 0x12, 0x02, 0x54, 0x01, 0x09, 0x12, 0x02, 0x54,
      0x01, 0x0a, 0x12, 0x02, 0x54, 0x01, 0x0b, 0x12, 0x02, 0x54, 0x01, 0x0c, 0x12, 0x02, 0x25, 0x01,
      0x03, 0x04, 0x12, 0x0c, 0x08, 0x04, 0x04, 0x08, 0x09, 0x06, 0x08, 0x0f, 0x03, 0x08, 0x05, 0x09,
      0x08, 0x0d, 0x0c, 0xff
   },

   {
      0x13, 0x14, 0x0f, 0x00, 0xaa, 0xbb, 0xcc, 0xdd, 0xee, 0x09, 0x09, 0x09, 0x09, 0x09, 0x14, 0x14,
      0x14, 0x14, 0x14, 0x41, 0x2f, 0x0f, 0x4f, 0x4f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x87, 0x00, 0x02, 0x28, 0x16, 0x07, 0x87, 0x00, 0x02, 0x14, 0x0c, 0x00, 0x25, 0x01, 0x03, 0x54,
      0x01, 0x04, 0x09, 0x04, 0x50, 0x02, 0x03, 0x09, 0x04, 0x48, 0x03, 0x03, 0x09, 0x04, 0x42, 0x02,
      0x0b, 0x02, 0x02, 0x04, 0x12, 0x0c, 0x42, 0x04, 0x04, 0x08, 0x04, 0xff
   },

   {
      0x14, 0x25, 0x0e, 0x14, 0x00, 0x00, 0x00, 0x00, 0x00, 0x07, 0x07, 0x07, 0x07, 0x07, 0x0a, 0x0a,
      0x0a, 0x0a, 0x0a, 0x41, 0x2f, 0x0f, 0x4f, 0x4f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x87, 0x00, 0x02, 0x28, 0x16, 0x07, 0x87, 0x00, 0x02, 0x14, 0x0c, 0x00, 0x25, 0x01, 0x04, 0x10,
      0x01, 0x03, 0x43, 0x01, 0x05, 0x07, 0x03, 0x41, 0x02, 0x04, 0x07, 0x03, 0x41, 0x02, 0x05, 0x07,
      0x03, 0x50, 0x02, 0x03, 0x06, 0x03, 0x04, 0x12, 0x0c, 0xff
   },

   /***
    * Boulder Dash 4
    */
   {
      0x01, 0x14, 0x05, 0x14, 0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x28, 0x28, 0x28, 0x2d, 0x32, 0x96, 0x8c,
      0x78, 0x64, 0x50, 0x06, 0x44, 0x3a, 0x06, 0x47, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x42, 0x11, 0x05, 0x08, 0x04, 0x42, 0x13, 0x05, 0x08, 0x04, 0x42, 0x0f, 0x0d, 0x07, 0x02, 0x42,
      0x0f, 0x0e, 0x04, 0x04, 0x42, 0x0f, 0x12, 0x07, 0x02, 0x42, 0x15, 0x0e, 0x04, 0x04, 0x40, 0x10,
      0x0e, 0x05, 0x02, 0x40, 0x10, 0x0f, 0x05, 0x02, 0x40, 0x10, 0x10, 0x05, 0x02, 0x40, 0x10, 0x11,
      0x05, 0x02, 0x50, 0x08, 0x03, 0x01, 0x01, 0x50, 0x23, 0x09, 0x01, 0x01, 0x50, 0x06, 0x11, 0x01,
      0x01, 0x70, 0x12, 0x05, 0x0d, 0x04, 0x25, 0x03, 0x03, 0x04, 0x26, 0x17, 0xff
   },

   {
      0x02, 0x14, 0x03, 0x05, 0x03, 0x00, 0x01, 0x57, 0x58, 0x63, 0x63, 0x63, 0x63, 0x63, 0x96, 0x8c,
      0x82, 0x78, 0x64, 0x06, 0x02, 0x3a, 0x06, 0x47, 0x01, 0x06, 0x10, 0x14, 0xfe, 0x40, 0x40, 0x25,
      0x42, 0x01, 0x03, 0x09, 0x03, 0x42, 0x0a, 0x0b, 0x09, 0x02, 0x42, 0x15, 0x0b, 0x09, 0x02, 0x42,
      0x1e, 0x0b, 0x09, 0x01, 0x42, 0x01, 0x16, 0x09, 0x01, 0x42, 0x0a, 0x0e, 0x09, 0x02, 0x42, 0x15,
      0x0e, 0x09, 0x02, 0x42, 0x1e, 0x0e, 0x09, 0x03, 0x70, 0x13, 0x0c, 0x02, 0x03, 0x40, 0x13, 0x0d,
      0x02, 0x01, 0x25, 0x0c, 0x03, 0x04, 0x19, 0x16, 0xff
   },

   {
      0x03, 0x14, 0x03, 0x05, 0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x5a, 0x5f, 0x63, 0x63, 0x63, 0x96, 0x96,
      0x96, 0x8c, 0x78, 0x01, 0x04, 0x3a, 0x06, 0x47, 0x00, 0x10, 0x14, 0x00, 0x3c, 0x32, 0x09, 0x00,
      0x3a, 0x02, 0x0e, 0x3a, 0x04, 0x06, 0x3a, 0x05, 0x13, 0x3a, 0x09, 0x0e, 0x3a, 0x0b, 0x09, 0x3a,
      0x0e, 0x14, 0x3a, 0x11, 0x0b, 0x3a, 0x14, 0x06, 0x3a, 0x14, 0x0f, 0x3a, 0x15, 0x14, 0x3a, 0x1a,
      0x07, 0x3a, 0x1a, 0x10, 0x3a, 0x21, 0x0b, 0x3a, 0x21, 0x13, 0x08, 0x03, 0x09, 0x08, 0x03, 0x0f,
      0x08, 0x09, 0x13, 0x08, 0x0f, 0x15, 0x08, 0x11, 0x0f, 0x08, 0x15, 0x07, 0x08, 0x1b, 0x07, 0x08,
      0x1b, 0x0b, 0x08, 0x25, 0x08, 0x08, 0x25, 0x15, 0x30, 0x06, 0x06, 0x70, 0x06, 0x0a, 0x02, 0x03,
      0x30, 0x07, 0x14, 0x30, 0x0b, 0x0e, 0x30, 0x0d, 0x09, 0x30, 0x11, 0x09, 0x30, 0x14, 0x04, 0x70,
      0x15, 0x0a, 0x03, 0x02, 0x70, 0x17, 0x14, 0x02, 0x01, 0x30, 0x18, 0x04, 0x30, 0x1c, 0x10, 0x30,
      0x23, 0x09, 0x30, 0x21, 0x10, 0x25, 0x0c, 0x03, 0x04, 0x26, 0x03, 0xff
   },

   {
      0x04, 0x14, 0x0a, 0x00, 0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x96, 0x82,
      0x6e, 0x5a, 0x5a, 0x05, 0x04, 0x07, 0x02, 0x47, 0x00, 0x10, 0x00, 0x00, 0x3c, 0x32, 0x09, 0x00,
      0x42, 0x07, 0x08, 0x08, 0x02, 0x42, 0x07, 0x09, 0x09, 0x04, 0x43, 0x08, 0x11, 0x06, 0x02, 0x42,
      0x0e, 0x09, 0x09, 0x04, 0xd0, 0x08, 0x09, 0x06, 0x02, 0xd0, 0x08, 0x0b, 0x06, 0x02, 0x50, 0x08,
      0x0d, 0x06, 0x02, 0x42, 0x19, 0x08, 0x08, 0x02, 0x42, 0x19, 0x09, 0x08, 0x04, 0x42, 0x19, 0x11,
      0x08, 0x02, 0x42, 0x20, 0x09, 0x08, 0x04, 0xd0, 0x1a, 0x09, 0x06, 0x02, 0xd0, 0x1a, 0x0b, 0x06,
      0x02, 0x50, 0x1a, 0x0d, 0x06, 0x02, 0x08, 0x24, 0x11, 0x10, 0x24, 0x12, 0x25, 0x03, 0x15, 0x04,
      0x27, 0x14, 0xff
   },

   {
      0x05, 0x14, 0x05, 0x0f, 0x5a, 0x5a, 0x5a, 0x5a, 0x5a, 0x55, 0x55, 0x55, 0x55, 0x55, 0x96, 0x96,
      0x96, 0x96, 0x96, 0x06, 0x07, 0x04, 0x05, 0x05, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x42, 0x26, 0x03, 0x09, 0x05, 0x42, 0x26, 0x16, 0x09, 0x07, 0x3a, 0x26, 0x06, 0x3a, 0x26, 0x06,
      0x10, 0x04, 0x0b, 0x10, 0x0a, 0x04, 0x10, 0x04, 0x0b, 0x08, 0x0a, 0x03, 0x08, 0x04, 0x0a, 0x25,
      0x22, 0x06, 0x04, 0x02, 0x17, 0xff
   },
   
   {
      0x06, 0x14, 0x0a, 0x14, 0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x14, 0x15, 0x16, 0x17, 0x18, 0x96, 0x96,
      0x8c, 0x82, 0x82, 0x01, 0x02, 0x00, 0x03, 0x06, 0x00, 0x10, 0x00, 0x00, 0x3c, 0x32, 0x09, 0x00,
      0x40, 0x01, 0x03, 0x26, 0x02, 0x40, 0x01, 0x04, 0x0a, 0x04, 0x40, 0x26, 0x04, 0x0a, 0x04, 0x40,
      0x03, 0x16, 0x22, 0x02, 0x54, 0x03, 0x04, 0x21, 0x02, 0x50, 0x02, 0x05, 0x23, 0x02, 0x41, 0x02,
      0x06, 0x0f, 0x04, 0x41, 0x02, 0x15, 0x21, 0x02, 0x41, 0x25, 0x06, 0x0f, 0x04, 0x48, 0x01, 0x0e,
      0x08, 0x04, 0x48, 0x26, 0x0e, 0x08, 0x04, 0x08, 0x02, 0x16, 0x14, 0x01, 0x16, 0x14, 0x26, 0x16,
      0x25, 0x18, 0x16, 0x04, 0x27, 0x03, 0xff
   },
   
   {
      0x07, 0x14, 0x05, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x5f, 0x5f, 0x5f, 0x5f, 0x5f, 0x96, 0x8c,
      0x82, 0x78, 0x64, 0x04, 0x06, 0x02, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x42, 0x06, 0x07, 0x08, 0x02, 0x42, 0x0d, 0x08, 0x09, 0x04, 0x42, 0x06, 0x10, 0x07, 0x02, 0x42,
      0x06, 0x08, 0x08, 0x04, 0x40, 0x05, 0x06, 0x0a, 0x02, 0x40, 0x0e, 0x07, 0x0b, 0x04, 0x40, 0x06,
      0x11, 0x09, 0x02, 0x40, 0x05, 0x06, 0x0b, 0x04, 0x42, 0x11, 0x07, 0x06, 0x02, 0x42, 0x16, 0x08,
      0x09, 0x04, 0x42, 0x11, 0x10, 0x05, 0x02, 0x42, 0x11, 0x08, 0x09, 0x04, 0x40, 0x10, 0x06, 0x08,
      0x02, 0x40, 0x17, 0x07, 0x0b, 0x04, 0x40, 0x16, 0x11, 0x07, 0x06, 0x40, 0x10, 0x07, 0x0a, 0x04,
      0x42, 0x1a, 0x07, 0x08, 0x02, 0x42, 0x21, 0x08, 0x09, 0x04, 0x42, 0x20, 0x10, 0x07, 0x06, 0x42,
      0x1a, 0x08, 0x08, 0x04, 0x40, 0x19, 0x06, 0x0a, 0x02, 0x40, 0x22, 0x07, 0x0b, 0x04, 0x40, 0x21,
      0x11, 0x09, 0x06, 0x40, 0x19, 0x07, 0x0a, 0x04, 0x10, 0x05, 0x04, 0x10, 0x17, 0x04, 0x10, 0x22,
      0x04, 0x3a, 0x09, 0x0b, 0x3a, 0x1d, 0x0b, 0x08, 0x05, 0x11, 0x08, 0x17, 0x11, 0x08, 0x22, 0x11,
      0x25, 0x13, 0x16, 0x04, 0x14, 0x0c, 0xff
   },
   
   {
      0x08, 0x14, 0x0f, 0x00, 0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x40, 0x40, 0x40, 0x40, 0x40, 0x96, 0x8c,
      0x82, 0x78, 0x64, 0x05, 0x44, 0x07, 0x03, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x54, 0x0b, 0x07, 0x11, 0x02, 0x54, 0x1b, 0x08, 0x0a, 0x04, 0x54, 0x0b, 0x08, 0x0a, 0x04, 0x54,
      0x0c, 0x11, 0x0f, 0x02, 0x54, 0x0d, 0x09, 0x0d, 0x02, 0x54, 0x19, 0x0a, 0x06, 0x04, 0x54, 0x0d,
      0x0f, 0x0c, 0x02, 0x54, 0x0d, 0x0a, 0x05, 0x04, 0x42, 0x0a, 0x06, 0x13, 0x02, 0x42, 0x0c, 0x08,
      0x0f, 0x02, 0x42, 0x0e, 0x0a, 0x0b, 0x02, 0x42, 0x0e, 0x0e, 0x0b, 0x02, 0x42, 0x0c, 0x10, 0x0f,
      0x02, 0x42, 0x0a, 0x12, 0x13, 0x02, 0x42, 0x0a, 0x06, 0x0d, 0x04, 0x42, 0x0c, 0x08, 0x09, 0x04,
      0x42, 0x0b, 0x0b, 0x03, 0x04, 0x42, 0x0d, 0x0b, 0x03, 0x04, 0x42, 0x0e, 0x0a, 0x05, 0x04, 0x42,
      0x18, 0x0a, 0x05, 0x04, 0x42, 0x1a, 0x08, 0x09, 0x04, 0x42, 0x19, 0x0b, 0x03, 0x04, 0x42, 0x1b,
      0x0b, 0x03, 0x04, 0x42, 0x1c, 0x06, 0x0d, 0x04, 0x40, 0x13, 0x06, 0x05, 0x04, 0x40, 0x13, 0x0e,
      0x05, 0x04, 0x40, 0x0a, 0x0c, 0x05, 0x02, 0x40, 0x18, 0x0c, 0x05, 0x02, 0x50, 0x03, 0x05, 0x04,
      0x03, 0x50, 0x20, 0x08, 0x04, 0x01, 0x08, 0x13, 0x06, 0x08, 0x0a, 0x0c, 0x08, 0x13, 0x12, 0x08,
      0x1c, 0x0c, 0x25, 0x13, 0x0c, 0x04, 0x13, 0x17, 0x40, 0x12, 0x07, 0x03, 0x02, 0x40, 0x12, 0x09,
      0x03, 0x02, 0x40, 0x12, 0x0f, 0x03, 0x02, 0x40, 0x12, 0x11, 0x03, 0x02, 0xff
   },
   
   {
      0x09, 0x14, 0x05, 0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x5a, 0x5a, 0x5a, 0x5a, 0x5a, 0xaa, 0xa0,
      0x96, 0x8c, 0x82, 0x06, 0x02, 0x05, 0x06, 0x06, 0x01, 0x06, 0x10, 0x14, 0xfe, 0x40, 0x40, 0x25,
      0x42, 0x05, 0x04, 0x13, 0x04, 0x42, 0x0b, 0x03, 0x12, 0x04, 0x42, 0x11, 0x0c, 0x0b, 0x04, 0x42,
      0x15, 0x03, 0x08, 0x04, 0x42, 0x1a, 0x03, 0x14, 0x04, 0x42, 0x1f, 0x04, 0x13, 0x04, 0x30, 0x03,
      0x05, 0x70, 0x08, 0x0a, 0x02, 0x03, 0x30, 0x0e, 0x10, 0x30, 0x10, 0x05, 0x30, 0x16, 0x10, 0x30,
      0x01, 0x13, 0x30, 0x0b, 0x15, 0x30, 0x16, 0x10, 0x30, 0x1c, 0x15, 0x30, 0x1f, 0x03, 0x30, 0x21,
      0x07, 0x30, 0x22, 0x0f, 0x30, 0x24, 0x12, 0x25, 0x03, 0x16, 0x04, 0x03, 0x17, 0xff
   },
   
   {
      0x0a, 0x14, 0x05, 0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x28, 0x28, 0x28, 0x28, 0x28, 0x96, 0x8c,
      0x82, 0x78, 0x6e, 0x02, 0x05, 0x07, 0x04, 0x06, 0x00, 0x00, 0x00, 0x00, 0xff, 0x00, 0x00, 0x00,
      0x42, 0x01, 0x07, 0x02, 0x02, 0x42, 0x02, 0x08, 0x0f, 0x04, 0x42, 0x25, 0x07, 0x02, 0x02, 0x42,
      0x25, 0x08, 0x0f, 0x04, 0x70, 0x01, 0x0d, 0x0a, 0x04, 0x70, 0x26, 0x0d, 0x0a, 0x04, 0x54, 0x0a,
      0x07, 0x14, 0x02, 0x41, 0x0a, 0x08, 0x14, 0x02, 0x41, 0x01, 0x04, 0x01, 0x01, 0x41, 0x26, 0x04,
      0x01, 0x01, 0x10, 0x01, 0x03, 0x10, 0x26, 0x03, 0x08, 0x03, 0x16, 0x08, 0x24, 0x16, 0x43, 0x0a,
      0x13, 0x09, 0x01, 0x43, 0x13, 0x0b, 0x02, 0x02, 0x43, 0x15, 0x0b, 0x09, 0x03, 0x25, 0x14, 0x05,
      0x04, 0x01, 0x17, 0x40, 0x0e, 0x04, 0x01, 0x01, 0x40, 0x12, 0x06, 0x01, 0x01, 0x40, 0x1b, 0x05,
      0x01, 0x01, 0x40, 0x1e, 0x09, 0x01, 0x01, 0x40, 0x18, 0x13, 0x01, 0x01, 0x40, 0x05, 0x0b, 0x01,
      0x01, 0xff
   },
   
   {
      0x0b, 0x14, 0x03, 0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x48, 0x48, 0x48, 0x48, 0x48, 0xb4, 0xb4,
      0xaa, 0xa0, 0x82, 0x03, 0x06, 0x04, 0x07, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x08, 0x05, 0x06, 0x08, 0x09, 0x06, 0x08, 0x0d, 0x06, 0x08, 0x11, 0x06, 0x08, 0x15, 0x06, 0x08,
      0x19, 0x06, 0x08, 0x1d, 0x06, 0x08, 0x21, 0x06, 0x14, 0x05, 0x07, 0x14, 0x09, 0x07, 0x14, 0x0d,
      0x07, 0x14, 0x11, 0x07, 0x14, 0x15, 0x07, 0x14, 0x19, 0x07, 0x14, 0x1d, 0x07, 0x14, 0x21, 0x07,
      0x30, 0x05, 0x0d, 0x30, 0x09, 0x0d, 0x30, 0x0d, 0x0d, 0x30, 0x11, 0x0d, 0x30, 0x15, 0x0d, 0x30,
      0x19, 0x0d, 0x30, 0x1d, 0x0d, 0x30, 0x21, 0x0d, 0x14, 0x05, 0x0e, 0x14, 0x09, 0x0e, 0x14, 0x0d,
      0x0e, 0x14, 0x11, 0x0e, 0x14, 0x15, 0x0e, 0x14, 0x19, 0x0e, 0x14, 0x1d, 0x0e, 0x14, 0x21, 0x0e,
      0x25, 0x01, 0x16, 0x04, 0x27, 0x08, 0xff
   },

   {
      0x0c, 0x14, 0x03, 0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x50, 0x50, 0x50, 0x50, 0x50, 0x96, 0x8c,
      0x82, 0x82, 0x6e, 0x07, 0x04, 0x01, 0x02, 0x03, 0x01, 0x06, 0x10, 0x14, 0xfe, 0x40, 0x40, 0x25,
      0x25, 0x01, 0x16, 0x04, 0x26, 0x03, 0xff
   },
   
   {
      0x0d, 0x14, 0x03, 0x05, 0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x32, 0x37, 0x3c, 0x41, 0x46, 0x96, 0x8c,
      0x82, 0x82, 0x6e, 0x06, 0x02, 0x07, 0x03, 0x05, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x3a, 0x12, 0x06, 0x3a, 0x12, 0x07, 0x42, 0x02, 0x10, 0x24, 0x02, 0x42, 0x02, 0x12, 0x24, 0x02,
      0x70, 0x02, 0x11, 0x23, 0x02, 0x25, 0x01, 0x16, 0x04, 0x26, 0x03, 0x40, 0x24, 0x11, 0x02, 0x02,
      0xff
   },
   
   {
      0x0e, 0x14, 0x14, 0x32, 0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x05, 0x05, 0x05, 0x05, 0x05, 0x6e, 0x6e,
      0x64, 0x5a, 0x5a, 0x06, 0x04, 0x01, 0x03, 0x02, 0x10, 0x14, 0x00, 0x00, 0xf0, 0x78, 0x00, 0x00,
      0x43, 0x01, 0x17, 0x26, 0x02, 0x41, 0x01, 0x16, 0x26, 0x02, 0x08, 0x09, 0x0a, 0x08, 0x01, 0x14,
      0x08, 0x14, 0x0a, 0x08, 0x0f, 0x0f, 0x08, 0x26, 0x09, 0x08, 0x09, 0x09, 0x08, 0x10, 0x04, 0x08,
      0x0d, 0x03, 0x25, 0x01, 0x16, 0x04, 0x26, 0x03, 0xff,
   },
   
   {
      0x0f, 0x14, 0x03, 0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x63, 0x63, 0x63, 0x63, 0x63, 0x96, 0x8c,
      0x82, 0x6e, 0x6e, 0x02, 0x06, 0x01, 0x04, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x54, 0x01, 0x03, 0x26, 0x02, 0x54, 0x26, 0x04, 0x12, 0x04, 0x54, 0x01, 0x04, 0x12, 0x04, 0x54,
      0x01, 0x16, 0x26, 0x02, 0x50, 0x0e, 0x08, 0x09, 0x02, 0x50, 0x0e, 0x09, 0x07, 0x04, 0x50, 0x0e,
      0x10, 0x09, 0x02, 0x50, 0x16, 0x09, 0x07, 0x04, 0x48, 0x0d, 0x07, 0x0b, 0x02, 0x48, 0x17, 0x08,
      0x09, 0x04, 0x48, 0x0d, 0x08, 0x09, 0x04, 0x48, 0x0d, 0x11, 0x0b, 0x02, 0x25, 0x13, 0x0b, 0x04,
      0x20, 0x0b, 0xff
   },
   
   {
      0x10, 0x14, 0x05, 0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x1e, 0x23, 0x28, 0x2d, 0x2d, 0x96, 0x8c,
      0x82, 0x6e, 0x6e, 0x03, 0x06, 0x01, 0x04, 0x03, 0x00, 0x10, 0x00, 0x00, 0x3c, 0x32, 0x09, 0x00,
      0x43, 0x03, 0x0e, 0x07, 0x02, 0x43, 0x0d, 0x14, 0x04, 0x02, 0x43, 0x1a, 0x07, 0x07, 0x02, 0x43,
      0x1a, 0x12, 0x05, 0x02, 0x25, 0x07, 0x04, 0x04, 0x26, 0x03, 0xff
   },
   
   {
      0x11, 0x14, 0x0a, 0x14, 0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x63, 0x63, 0x63, 0x63, 0x63, 0x1e, 0x1e,
      0x1e, 0x1e, 0x1e, 0x02, 0x03, 0x03, 0x05, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x87, 0x00, 0x02, 0x28, 0x16, 0x07, 0x87, 0x00, 0x02, 0x14, 0x0c, 0x01, 0x54, 0x01, 0x03, 0x12,
      0x02, 0x54, 0x01, 0x04, 0x12, 0x02, 0x54, 0x01, 0x05, 0x12, 0x02, 0x54, 0x01, 0x06, 0x12, 0x02,
      0x54, 0x01, 0x07, 0x12, 0x02, 0x54, 0x01, 0x08, 0x12, 0x02, 0x54, 0x01, 0x09, 0x12, 0x02, 0x54,
      0x01, 0x0a, 0x12, 0x02, 0x54, 0x01, 0x0b, 0x12, 0x02, 0x54, 0x01, 0x0c, 0x12, 0x02, 0x08, 0x03,
      0x05, 0x08, 0x03, 0x0a, 0x08, 0x07, 0x08, 0x08, 0x0e, 0x08, 0x08, 0x0b, 0x0a, 0x25, 0x01, 0x03,
      0x04, 0x12, 0x03, 0xff
   },
 
   {
      0x12, 0x14, 0x0a, 0x14, 0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x03, 0x03, 0x03, 0x03, 0x03, 0x14, 0x14,
      0x14, 0x14, 0x14, 0x02, 0x03, 0x03, 0x05, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x87, 0x00, 0x02, 0x28, 0x16, 0x07, 0x87, 0x00, 0x02, 0x14, 0x0c, 0x01, 0x50, 0x02, 0x03, 0x05,
      0x03, 0x50, 0x01, 0x05, 0x05, 0x03, 0x50, 0x07, 0x07, 0x03, 0x02, 0x50, 0x06, 0x09, 0x04, 0x02,
      0x48, 0x0b, 0x03, 0x03, 0x02, 0x50, 0x0b, 0x04, 0x03, 0x02, 0x43, 0x0b, 0x06, 0x03, 0x02, 0x43,
      0x0b, 0x08, 0x03, 0x02, 0x43, 0x0b, 0x0a, 0x03, 0x02, 0x25, 0x01, 0x03, 0x04, 0x01, 0x0b, 0xff
   },
 
   {
      0x13, 0x14, 0x0a, 0x14, 0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x04, 0x04, 0x04, 0x04, 0x04, 0x0a, 0x0a,
      0x0a, 0x0a, 0x0a, 0x02, 0x03, 0x03, 0x05, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x87, 0x00, 0x02, 0x28, 0x16, 0x07, 0x87, 0x00, 0x02, 0x14, 0x0c, 0x00, 0x54, 0x06, 0x07, 0x05,
      0x02, 0x54, 0x08, 0x05, 0x05, 0x04, 0x08, 0x08, 0x06, 0x08, 0x08, 0x08, 0x08, 0x09, 0x07, 0x08,
      0x07, 0x07, 0x25, 0x01, 0x03, 0x04, 0x01, 0x0b, 0xff
   },

   {
      0x14, 0x14, 0x63, 0x00, 0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x01, 0x01, 0x01, 0x01, 0x01, 0x0a, 0x0a,
      0x0a, 0x0a, 0x0a, 0x05, 0x06, 0x02, 0x03, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x87, 0x00, 0x02, 0x28, 0x16, 0x07, 0x87, 0x00, 0x02, 0x14, 0x0c, 0x00, 0x41, 0x01, 0x03, 0x12,
      0x02, 0x41, 0x01, 0x04, 0x12, 0x02, 0x41, 0x01, 0x05, 0x12, 0x02, 0x41, 0x01, 0x06, 0x12, 0x02,
      0x41, 0x01, 0x07, 0x12, 0x02, 0x41, 0x01, 0x08, 0x12, 0x02, 0x41, 0x01, 0x09, 0x12, 0x02, 0x41,
      0x01, 0x0a, 0x12, 0x02, 0x41, 0x01, 0x0b, 0x12, 0x02, 0x41, 0x01, 0x0c, 0x12, 0x02, 0x42, 0x04,
      0x07, 0x01, 0x01, 0x40, 0x04, 0x08, 0x01, 0x01, 0x43, 0x0a, 0x07, 0x01, 0x01, 0x40, 0x0a, 0x08,
      0x01, 0x01, 0x10, 0x07, 0x04, 0x25, 0x01, 0x03, 0x04, 0x01, 0x0c, 0xff
   }
};


#ifdef ANSI_FUNC

void commodore_change_colors (struct CAVE_RAW_DATA *cave_raw)
#else

void commodore_change_colors (cave_raw)
   struct CAVE_RAW_DATA *cave_raw;
#endif
{
   static struct
   {
      unchar  object;
      short   size;
      short   color[3];
      unchar *image[32];
   } object_data[] =
      {
         {         DIRT, 32 * 32, {  2, 11, -1 }, { commodore_dirt, NULL } },
         {   BRICK_WALL, 32 * 32, {  2, 11,  1 }, { commodore_brick_wall, NULL } },
         {   MAGIC_WALL, 32 * 32, { -1, 11,  1 }, { commodore_magic_wall_0, commodore_magic_wall_1, commodore_magic_wall_2, commodore_magic_wall_3, NULL } },
         { FLASH_OUTBOX, 32 * 32, { -1, 11, -1 }, { commodore_cave_out, NULL } },
         {         WALL, 32 * 32, {  2, 11, -1 }, { commodore_steel_wall, NULL } },
         {    FIREFLY_1, 32 * 32, { -1, 11, -1 }, { commodore_firefly_0, commodore_firefly_1, commodore_firefly_2, commodore_firefly_3, NULL } },
         {      BOULDER, 32 * 32, {  2, 11,  1 }, { commodore_boulder, NULL } },
         {      DIAMOND, 32 * 32, {  2, 11, -1 }, { commodore_diamond_0, commodore_diamond_1, commodore_diamond_2, commodore_diamond_3, commodore_diamond_4, commodore_diamond_5, commodore_diamond_6, commodore_diamond_7, NULL } },
         {   EXPLODE_S0, 32 * 32, {  2, 11, -1 }, { commodore_explode_s0, commodore_explode_s1, commodore_explode_s2, NULL } },
         {   EXPLODE_D0, 32 * 32, {  2, 11, -1 }, { commodore_explode_d0, commodore_explode_d1, commodore_explode_d2, commodore_explode_d3, NULL } },
         {  BUTTERFLY_1, 32 * 32, {  2, 11, -1 }, { commodore_butterfly_0, commodore_butterfly_1, commodore_butterfly_2, NULL } },
         {     ROCKFORD, 32 * 32, {  2, 11,  1 }, { commodore_rockford_0, commodore_rockford_1, commodore_rockford_2, commodore_rockford_3, commodore_rockford_4, commodore_rockford_5, commodore_rockford_6, commodore_rockford_7, commodore_rockford_8, commodore_rockford_l0, commodore_rockford_l1, commodore_rockford_l2, commodore_rockford_l3, commodore_rockford_l4, commodore_rockford_l5, commodore_rockford_l6, commodore_rockford_r0, commodore_rockford_r1, commodore_rockford_r2, commodore_rockford_r3, commodore_rockford_r4, commodore_rockford_r5, commodore_rockford_r6, NULL } },
         {       AMOEBA, 32 * 32, { -1, 11, -1 }, { commodore_amoeba_0, commodore_amoeba_1, commodore_amoeba_2, commodore_amoeba_3, NULL } },
         {    WALL_PART, 16 * 16, {  2, 11, -1 }, { commodore_wall_s0, commodore_wall_s1, commodore_wall_s2, commodore_wall_s3, commodore_wall_s4, commodore_wall_s5, commodore_wall_s6, commodore_wall_s7, NULL } },
         {   EXPLODE_S0, 32 * 32, {  9, 11, -1 }, { commodore_explode_s0, commodore_explode_s1, commodore_explode_s2, NULL } },
         {   EXPLODE_D0, 32 * 32, {  9, 11, -1 }, { commodore_explode_d0, commodore_explode_d1, commodore_explode_d2, commodore_explode_d3, NULL } },
         {            0, 32 * 32, {  0,  0,  0 }, { NULL } }
      };
   short n, i, j, c, color[3];

   debug (("{%s(0x%08lx)", __FUNCTION__, (long) cave_raw))
   debug (("0x%02x 0x%02x 0x%02x 0x%02x 0x%02x\n", cave_raw->bg1_color, cave_raw->bg2_color, cave_raw->fg_color, cave_raw->unused[0], cave_raw->unused[1]))

   color[0] = cave_raw->bg1_color;
   color[1] = cave_raw->bg2_color;
   color[2] = (cave_raw->fg_color == 9) ? 1 : 5;   /* Nesto kao da paleta ne odgovara... */

   for (n = 0; object_data[n].image[0]; n++)
   {
      for (i = 0; object_data[n].image[i]; i++)
         for (j = 0; j < object_data[n].size; j++)
            for (c = 0; c < 3; c++)
               if (object_data[n].image[i][j] == (object_data[n].color[c] + PALETTE_COMMODORE_OFFSET))
               {
                  object_data[n].image[i][j] = color[c] + PALETTE_COMMODORE_OFFSET;

                  break;
               }

      for (c = 0; c < 3; c++)
         object_data[n].color[c] = color[c];
   }

   debug_endfunc
}
